/* Generated automatically. */
static const char configuration_arguments[] = "/var/lib/managarm-buildenv/src/ports/gcc/configure --with-pkgversion=managarm --with-bugurl=https://github.com/managarm/bootstrap-managarm/issues/ --prefix=/var/lib/managarm-buildenv/build/tools/bootstrap-system-gcc --target=x86_64-linux-mlibc --with-sysroot=/var/lib/managarm-buildenv/build/system-root --without-headers --without-newlib --enable-languages=c,c++,lto --disable-fixincludes --disable-shared --disable-hosted-libstdcxx --disable-wchar_t --disable-libssp --disable-libsanitizer --disable-libquadmath --enable-linker-build-id CFLAGS=-O2 CXXFLAGS=-O2";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "generic" }, { "arch", "x86-64" } };
